//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Class          : TypedBufferReader
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon UniversityF
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
// Objects constructed from the TypedBufferReader class allow
// information to be read from files. The programmer must know the
// type of each datum read, in order to read it correctly.
//
// A primitive error-correction mechanism allows for retrieval of any
// datum that is not correctly parsed.   
//
// Future Plans   : Add a constructor for specifying a file name directly
//
// Program History:
//   5/18/01: R. Pattis - Operational for 15-100
//   8/20/01: R. Pattis - JavaDoc comments added
//   6.02.02: R. Pattis - Ignore leading/trailing quotes in file names
//                          added by new Microsoft OS
//
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


package edu.cmu.cs.pattis.cs151xx;

import edu.cmu.cs.pattis.cs151xx.Prompt;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.EOFException;
import java.io.IOException;

import java.util.StringTokenizer;





/** 
 * Objects constructed from the <code>TypedBufferReader</code> class allow
 *   information to be read from files.
 * The programmer must know the type of each datum read, in order to read
 *   it correctly.
 * A primitive error-correction mechanism allow for retrieval of any datum
 *   that is not correctly parsed.   
 * 
 * @author Richard E. Pattis (Computer Science Department, Carnegie Mellon)
*/
public class TypedBufferReader {
  

	/** 
	 * Constructs a <code>TypedBufferReader</code> object, specifying the
	 *   prompt (for a file name) to the user; if the user specifies a
	 *   file name that cannot be located, he/she is reprompted.
	 *
	 * @param  prompt specifies the prompt to the user to enter a file name
	 *         if the first character is a quote, the rest of the characters
	 *         will be used as the name of the file (not a prompt).
	*/
  public TypedBufferReader (String prompt)
  {this(prompt,"");} 
   
  

	/** 
	 * Constructs a <code>TypedBufferReader</code> object, specifying the
	 *   prompt (for a file name) to the user (and what is considered
	 *   whitespace); if the user specifies a file name that cannot be
	 *   located, he/she is reprompted.
	 *
	 * @param  prompt specifies the prompt to the user to enter a file name
	 *         if the first character is a quote, the rest of the characters
	 *         will be used as the name of the file (not a prompt).
	 * @param  extraWhitespace specifies any other characters in the file to
	 *         consider whitespace (they separate tokens, but otherwise are
	 *         ignored)
	*/
  public TypedBufferReader (String prompt, String extraWhitespace)
  { 
    boolean direct = prompt.charAt(0)=='\"';
    this.whitespace = " \t\n\r\f" + extraWhitespace;
    for (;;)
      try {
        fileName = (direct ? prompt.substring(1) : Prompt.forString(prompt));
        fileName = stripQuotes(fileName);
        br       = new BufferedReader( new FileReader(fileName) );
        break;
      }catch (Exception e)
         {
            if (!direct)
              System.out.println("File not found; reenter");
            else {
              System.out.println(prompt + "\" not found; reenter");
              prompt = "Enter file name";
              direct = false;
            }
         }
    currentLineRaw = "";
    currentLine    = new StringTokenizer("",whitespace);
    lineNumber     = 0;
  }
 
  

	/** 
	 * Constructs a <code>TypedBufferReader</code> object, specifying an
	 *   <code>InputStream</code> (e.g., <code>System.in</code>).
	 *
	 * @param  is specifies an input stream to read from (e.g.,
	 *         <code>System.in</code> to read from the keyboard).
*/
  public TypedBufferReader (InputStream is)
  {this(is,"");}
   

  
	/** 
	 * Constructs a <code>TypedBufferReader</code> object, specifying an
	 *   <code>InputStream</code> (e.g., <code>System.in</code>) and what
	 *   is considered whitespace.
	 *
	 * @param  is specifies an input stream to read from (e.g.,
	 *         <code>System.in</code> to read from the keyboard).
	 * @param  extraWhitespace specifies any other characters in the file to
	 *         consider whitespace (they separate tokens, but otherwise are
	 *         ignored)
*/
  public TypedBufferReader (InputStream is, String extraWhitespace)
  {
    this.whitespace = " \t\n\r\f"+extraWhitespace;
  	fileName       = "some-InputStream";
    br             = new BufferedReader( new InputStreamReader(is));
    currentLineRaw = "";
    currentLine    = new StringTokenizer("",whitespace);
    lineNumber     = 0;
  }
  
  
  
  
  
	/** 
	 * Returns the file name that this this object is connected to.
	 *
	 * @return the file name that this object is connected to (or
	 *         "some-InputStream")
	*/
  public String getFileName ()
  {return fileName;}
  
  
  
  
  
	/** 
	 * Returns the line number currently being processed in the file that
	 *   this object is connected to.
	 *
	 * @return the line number currently being processed in the file that
	 *   this object is connected to (useful when errors are detected in
	 *   a file); note this line number is intially 0, and is the line
	 *   number after the last read (the next read token may be on the
	 *   following line)
	*/
  public int getLineNumber ()
  {return lineNumber;}
  
  
  
  
  
	/** 
	 * Returns the next token (when there is one) uninterpreted (as a 
	 *   <code>String</code>).
	 *
	 * @return the next token (when there is one) uninterpreted (as a 
	 *   <code>String</code>); otherwise throw an exception
	 *
	 * @throws EOFException if there are no more tokens to read from the file
	*/
  private String readNextToken () throws EOFException
  {
    
    for (;;)
      if (currentLine.hasMoreTokens()) {
        untypedToken = currentLine.nextToken();
        currentLineRaw = currentLineRaw.substring(currentLineRaw.indexOf(untypedToken)+untypedToken.length());
        return untypedToken;
      }else try {
        if (currentLineRaw == null)
          throw new EOFException();
        currentLineRaw = br.readLine();
        lineNumber++;
        if (currentLineRaw == null)
          throw new EOFException();
        currentLine = new StringTokenizer(currentLineRaw,whitespace);
      } catch (IOException eof) {throw new EOFException();}
  }
  
    

	/** 
	 * Returns a <code>String</code> (not a token, it may contain white space) containing
	 *   everything <b>remaining</b> on the current line (everything on the line, if called
	 *   before any tokens are read from the line).
	 *
	 * @return a <code>String</code> (not a token) containing everything left on
	 *   the current line (note the difference between this method and
	 *   <code>readString</code>); otherwise throw an exception.
	 * <p>
	 * Note that if the previous read was a token read from the current line, white space
	 *   following that token will be included as part of the <code>String</code>
	 *   returned by this method.
	 *
	 * @throws EOFException if there are no more lines to read from the file
	*/
  public String readLine () throws EOFException
  {
    
    if (currentLineRaw == null)
      throw new EOFException();
      
    if (currentLineRaw.equals(""))
      try {
        currentLineRaw = br.readLine();
        lineNumber++;
        if (currentLineRaw == null)
          throw new EOFException();
    } catch (IOException eof) {throw new EOFException();}
      
    String answer  = currentLineRaw;
    currentLineRaw = "";
    currentLine    = new StringTokenizer("",whitespace);
    
    return answer;
    
  }
  
    

	/** 
	 * Returns the next token (when there is one) interpreted as
	 *   an <code>int</code>.
	 *
	 * @return  the next token (when there is one) interpreted as
	 *   an <code>int</code>; otherwise throw an exception
	 *
	 * @throws EOFException if there are no more tokens to read from the file
	 * @throws NumberFormatException if the next token is not parsable as
	 *         an <code>int</code>
	*/
  public int readInt () throws NumberFormatException, EOFException
  {return Integer.parseInt(readNextToken());}
     
 
   
    

	/** 
	 * Returns the next token (when there is one) interpreted as
	 *   a <code>double</code>.
	 *
	 * @return  the next token (when there is one) interpreted as
	 *   a <code>double</code>; otherwise throw an exception
	 *
	 * @throws EOFException if there are no more tokens to read from the file
	 * @throws NumberFormatException if the next token is not parsable as
	 *         a <code>double</code>
	*/
  public double readDouble () throws NumberFormatException, EOFException
  {return Double.parseDouble(readNextToken());}
     
 
   
    

	/** 
	 * Returns the next token (when there is one) interpreted as
	 *   a <code>String</code>.
	 *
	 * @return the next token (when there is one) interpreted as
	 *   a <code>String</code>; otherwise throw an exception
	 *
	 * @throws EOFException if there are no more tokens to read from the file
	*/
  public String readString () throws EOFException
  {return readNextToken();}
     
 
   
    

	/** 
	 * Returns the next token (when there is one) interpreted as
	 *   a <code>boolean</code>.
	 *
	 * @return  the next token (when there is one) interpreted as
	 *   a <code>boolean</code> (either <code>true</code> or
	     <code>false</code>, case-sensitive!); otherwise throw an exception
	 *
	 * @throws EOFException if there are no more tokens to read from the file
	 * @throws NumberFormatException if the next token is not parsable as
	 *         a <code>boolean</code>
	*/
  public boolean readBoolean () throws NumberFormatException, EOFException
  {
    String temp = readNextToken();
    if (temp.equals("true"))
      return true;
    if (temp.equals("false"))
      return false;
    throw new NumberFormatException();
  }
     
 
   
    

	/** 
	 * Returns the token (as a <code>String</code>) just read.
	 *
	 * @return  the token (as a <code>String</code>) just read; this
	 *          method is mostly used to try to recover from a
	 *          <code>NumberFormatException</code>, by allowing the
	 *          program to examine the token read, without trying to
	 *          parse it (since the parsing failed)
	*/
  public String getLastTokenUntyped ()
  {return untypedToken;}

 
   
    

	/** 
	 * Ignores all the remaining tokens on the current line (useful for
	 *   error processing).
	*/
  public void ignoreRestOfLine() 
  {currentLine = new StringTokenizer("",whitespace);}

 
   
    

	/** 
	 * Returns one giant <code>String</code> containing all the (remaining)
	 *   tokens in the <code>TypedBufferReader</code>, separated by one
	 *   space.
	 * Typically, the result will be scanned with a
	 *   <code>StringTokenizer</code>
	 *
	 * @return one giant <code>String</code> containing all the (remaining)
	 *   tokens in the <code>TypedBufferReader</code>, separated by one
	 *   space.
	*/
	public String readAllTokens()
	{
	   StringBuffer result = new StringBuffer();
	   for (;;)
	     try {
	       result.append(readString() + " ");
	     } catch (EOFException eofe) {return result.toString();}
	}



	/** 
	 * Closes an open <code>TypedBufferReader</code>, disconnecting it
	 *   from its source.
	*/
  public void close() 
  {
    try {
      br.close();
		}
 	  catch (IOException ioe) {}
  }
  
 

	/** 
	 * Strips all the quotes from its argument.
	 *
	 * @param  s specifies a file name
	 *
	 * @return  a <code>String</code> containing all characters in <code>s</code> except the
	 *          quotes
	*/
  private static String stripQuotes(String s) 
  {
    String answer = "";
    for (int i=0; i<s.length(); i++)
      if (s.charAt(i) != '"')
        answer += s.charAt(i);
        
    return answer;
  }
  
 




	/** 
	 * Buffered reader (set to a file or an <code>InputStream</code>).
	*/
  private BufferedReader  br;
  
  
	/** 
	 * Name of file being processed.
	*/
  private String fileName;
  
  
	/** 
	 * Characters considered whitespace.
	*/
  private String whitespace;
  
  
	/** 
	 * How many lines have been read from the file (useful for error recovery).
	*/
  private int lineNumber;
  
  
	/** 
	 * Stores the current line being parsed by the string tokenizer.
	*/
  private StringTokenizer currentLine;
  
  
	/** 
	 * Stores the current line as a String; as each token is gotten, it it
	 *   removed from so only the "rest" of the line is present.
	*/
  private String currentLineRaw;
  
  
	/** 
	 * Stores the last token parsed (which may have failed to be parsed
	 *   correctly) by the string tokenizer.
	*/
  private String untypedToken = "";
}


